package com.sooka.gateway.filter;

import com.alibaba.fastjson.JSONObject;
import com.sooka.enums.ResultEnum;
import com.sooka.gateway.feign.DeptFeignService;
import com.sooka.model.ResultModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.cloud.gateway.support.ServerWebExchangeUtils;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.validation.constraints.NotNull;
import java.net.URI;
import java.nio.charset.StandardCharsets;
import java.util.*;

/******************************
 * TODO
 * @author yanhongliang
 * @date 2020-07-20 16:07
 ******************************/

@RefreshScope
@Component
public class AuthAndLogFilter implements GlobalFilter {

    @Value("${gate.ignore.startWith}")
    private String startWith;

    @Autowired
    private DeptFeignService deptFeignService;


    @Override
    public Mono<Void> filter(ServerWebExchange serverWebExchange, GatewayFilterChain gatewayFilterChain) {

        LinkedHashSet requiredAttribute = serverWebExchange.getRequiredAttribute(ServerWebExchangeUtils.GATEWAY_ORIGINAL_REQUEST_URL_ATTR);
        ServerHttpRequest request = serverWebExchange.getRequest();
        String requestUri = request.getPath().pathWithinApplication().value();

        if (requiredAttribute != null) {
            for (Object o : requiredAttribute) {
                URI next = (URI) o;
                requestUri = next.getPath();
            }
        }

        //请求方式
        final String method = Objects.requireNonNull(request.getMethod()).toString();
        ServerHttpRequest.Builder mutate = request.mutate();
        ServerHttpRequest build = mutate.build();

        // 不进行拦截的地址
        if (isStartWith(requestUri)) {
            return gatewayFilterChain.filter(serverWebExchange.mutate().request(build).build());
        }

        List<String> secretKeys = request.getHeaders().get("secretKey");
        String secretKey = null;
        if (secretKeys != null) {
            secretKey = secretKeys.get(0);
        }

        if (secretKey == null) {
            return getVoidMono(serverWebExchange, ResultModel.error(ResultEnum.ERROR_INVALID_SECRET_KEY));
        }

        List<String> intCodes = request.getHeaders().get("intCode");
        String intCode = null;
        if (intCodes != null) {
            intCode = intCodes.get(0);
        }

        if (intCode == null) {
            return getVoidMono(serverWebExchange, ResultModel.error(ResultEnum.ERROR_INT_CODE));
        }

        // 用accessId查询接口ID、secretKey、使用时限
        // 判断accessId是否在有效期内
        ResultModel model = deptFeignService.test(secretKey, intCode);

        if (!ResultEnum.OK.getCode().equals(model.getCode())) {
            return getVoidMono(serverWebExchange, model);
        }

        return gatewayFilterChain.filter(serverWebExchange.mutate().request(build).build());
    }

    /**
     * 网关抛异常
     */
    @NotNull
    private Mono<Void> getVoidMono(ServerWebExchange serverWebExchange, ResultModel model) {
        serverWebExchange.getResponse().setStatusCode(HttpStatus.OK);
        byte[] bytes = JSONObject.toJSONString(model).getBytes(StandardCharsets.UTF_8);

        DataBuffer buffer = serverWebExchange.getResponse().bufferFactory().wrap(bytes);
        return serverWebExchange.getResponse().writeWith(Flux.just(buffer));
    }

    /**
     * URI是否以什么打头
     */
    private boolean isStartWith(String requestUri) {
        boolean flag = false;
        for (String s : startWith.split(",")) {
            if (requestUri.startsWith(s)) {
                flag = true;
            }
        }
        return flag;
    }


}
